@@ -1,49 +1,21 @@
 /*
+ * @copyright Copyright (c) OX Software GmbH, Germany <info@open-xchange.com>
+ * @license AGPL-3.0
  *
- *    OPEN-XCHANGE legal information
+ * This code is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU Affero General Public License as published by
+ * the Free Software Foundation, either version 3 of the License, or
+ * (at your option) any later version.
  *
- *    All intellectual property rights in the Software are protected by
- *    international copyright laws.
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU Affero General Public License for more details.
  *
+ * You should have received a copy of the GNU Affero General Public License
+ * along with OX App Suite.  If not, see <https://www.gnu.org/licenses/agpl-3.0.txt>.
  *
- *    In some countries OX, OX Open-Xchange, open xchange and OXtender
- *    as well as the corresponding Logos OX Open-Xchange and OX are registered
- *    trademarks of the OX Software GmbH group of companies.
- *    The use of the Logos is not covered by the GNU General Public License.
- *    Instead, you are allowed to use these Logos according to the terms and
- *    conditions of the Creative Commons License, Version 2.5, Attribution,
- *    Non-commercial, ShareAlike, and the interpretation of the term
- *    Non-commercial applicable to the aforementioned license is published
- *    on the web site http://www.open-xchange.com/EN/legal/index.html.
- *
- *    Please make sure that third-party modules and libraries are used
- *    according to their respective licenses.
- *
- *    Any modifications to this package must retain all copyright notices
- *    of the original copyright holder(s) for the original code used.
- *
- *    After any such modifications, the original and derivative code shall remain
- *    under the copyright of the copyright holder(s) and/or original author(s)per
- *    the Attribution and Assignment Agreement that can be located at
- *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
- *    given Attribution for the derivative code and a license granting use.
- *
- *     Copyright (C) 2016-2020 OX Software GmbH
- *     Mail: info@open-xchange.com
- *
- *
- *     This program is free software; you can redistribute it and/or modify it
- *     under the terms of the GNU General Public License, Version 2 as published
- *     by the Free Software Foundation.
- *
- *     This program is distributed in the hope that it will be useful, but
- *     WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
- *     or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
- *     for more details.
- *
- *     You should have received a copy of the GNU General Public License along
- *     with this program; if not, write to the Free Software Foundation, Inc., 59
- *     Temple Place, Suite 330, Boston, MA 02111-1307 USA
+ * Any use of the work other than as authorized under this license or copyright law is prohibited.
  *
  */
 
@@ -54,7 +26,6 @@ import java.io.IOException;
 import java.util.Collection;
 import java.util.HashMap;
 import java.util.Map;
-import java.util.UUID;
 import javax.servlet.http.Cookie;
 import javax.servlet.http.HttpServletRequest;
 import javax.servlet.http.HttpServletResponse;
@@ -74,7 +45,6 @@ import com.openexchange.groupware.contexts.Context;
 import com.openexchange.groupware.contexts.impl.ContextExceptionCodes;
 import com.openexchange.groupware.notify.hostname.HostnameService;
 import com.openexchange.java.Strings;
-import com.openexchange.java.util.UUIDs;
 import com.openexchange.log.LogProperties;
 import com.openexchange.login.LoginRequest;
 import com.openexchange.login.LoginResult;
@@ -92,8 +62,6 @@ import com.openexchange.session.reservation.Reservation;
 import com.openexchange.session.reservation.SessionReservationService;
 import com.openexchange.sessiond.SessionFilter;
 import com.openexchange.sessiond.SessiondService;
-import com.openexchange.tools.servlet.http.Cookies;
-import com.openexchange.tools.servlet.http.Tools;
 import com.openexchange.tools.session.ServerSessionAdapter;
 import com.openexchange.user.User;
 import com.openexchange.user.UserService;
@@ -208,11 +176,8 @@ public class SAMLLoginRequestHandler implements LoginRequestHandler {
             }
         }
 
-        // open-xchange-saml-<hash> cookie Value to lookup related session
-        String samlCookieValue = UUIDs.getUnformattedString(UUID.randomUUID());
-
         // Do the login
-        LoginResult result = login(req, context, user, reservation.getState(), conf, samlCookieValue);
+        LoginResult result = login(req, context, user, reservation.getState(), conf);
 
         // Obtain associated session
         Session session = result.getSession();
@@ -227,9 +192,8 @@ public class SAMLLoginRequestHandler implements LoginRequestHandler {
         SessionUtility.rememberSession(req, new ServerSessionAdapter(session));
         LoginServlet.writeSecretCookie(req, resp, session, session.getHash(), req.isSecure(), req.getServerName(), conf);
 
-        // Set SAML cookie
-        Cookie samlSessionCookie = configureSAMLCookie(req, samlCookieValue, session);
-        resp.addCookie(samlSessionCookie);
+        // Set session cookie
+        LoginServlet.writeSessionCookie(resp, session, session.getHash(), req.isSecure(), req.getServerName());
 
         // Send redirect
         String uiWebPath = req.getParameter(SAMLLoginTools.PARAM_LOGIN_PATH);
@@ -241,21 +205,7 @@ public class SAMLLoginRequestHandler implements LoginRequestHandler {
         resp.sendRedirect(SAMLLoginTools.buildFrontendRedirectLocation(session, uiWebPath, uriFragment));
     }
 
-    private Cookie configureSAMLCookie(HttpServletRequest req, String samlCookieValue, Session session) {
-        boolean isHttps = Tools.considerSecure(req);
-        String hostName = SAMLLoginTools.getHostName(services.getOptionalService(HostnameService.class), req);
-        Cookie samlSessionCookie = new Cookie(SAMLLoginTools.AUTO_LOGIN_COOKIE_PREFIX + session.getHash(), samlCookieValue);
-        samlSessionCookie.setPath("/");
-        samlSessionCookie.setSecure(isHttps);
-        samlSessionCookie.setMaxAge(-1);
-        String cookieDomain = Cookies.getDomainValue(hostName);
-        if (cookieDomain != null) {
-            samlSessionCookie.setDomain(cookieDomain);
-        }
-        return samlSessionCookie;
-    }
-
-    protected LoginResult login(HttpServletRequest httpRequest, final Context context, final User user, final Map<String, String> optState, LoginConfiguration loginConfiguration, final String samlCookieValue) throws OXException {
+    protected LoginResult login(HttpServletRequest httpRequest, final Context context, final User user, final Map<String, String> optState, LoginConfiguration loginConfiguration) throws OXException {
         // The properties derived from optional state
         final Map<String, Object> props = optState == null ? new HashMap<String, Object>(4) : new HashMap<String, Object>(optState);
 
@@ -267,7 +217,7 @@ public class SAMLLoginRequestHandler implements LoginRequestHandler {
         LoginResult loginResult = LoginPerformer.getInstance().doLogin(loginRequest, props, new LoginMethodClosure() {
             @Override
             public Authenticated doAuthentication(LoginResultImpl retval) throws OXException {
-                Authenticated authenticated = enhanceAuthenticated(new AuthenticatedImpl(context.getLoginInfo()[0], user.getLoginInfo()), samlCookieValue, optState);
+                Authenticated authenticated = enhanceAuthenticated(new AuthenticatedImpl(context.getLoginInfo()[0], user.getLoginInfo()), optState);
                 return authenticated;
             }
         });
@@ -275,7 +225,7 @@ public class SAMLLoginRequestHandler implements LoginRequestHandler {
         return loginResult;
     }
 
-    Authenticated enhanceAuthenticated(Authenticated authenticated, final String samlCookieValue, final Map<String, String> reservationState) {
+    Authenticated enhanceAuthenticated(Authenticated authenticated, final Map<String, String> reservationState) {
         if (reservationState != null) {
             String samlAuthenticated = reservationState.get(SAMLSessionParameters.AUTHENTICATED);
             if (samlAuthenticated != null && Boolean.parseBoolean(samlAuthenticated)) {
@@ -303,10 +253,6 @@ public class SAMLLoginRequestHandler implements LoginRequestHandler {
                             session.setParameter(SAMLSessionParameters.SESSION_NOT_ON_OR_AFTER, sessionNotOnOrAfter);
                         }
 
-                        if (samlCookieValue != null) {
-                            session.setParameter(SAMLSessionParameters.SESSION_COOKIE, samlCookieValue);
-                        }
-
                         String singleLogout = reservationState.get(SAMLSessionParameters.SINGLE_LOGOUT);
                         if (singleLogout != null) {
                             session.setParameter(SAMLSessionParameters.SINGLE_LOGOUT, Boolean.valueOf(singleLogout));
@@ -372,24 +318,31 @@ public class SAMLLoginRequestHandler implements LoginRequestHandler {
     private String tryAutoLogin(HttpServletRequest httpRequest, HttpServletResponse httpResponse, Reservation reservation, SAMLBackend samlBackend) throws OXException {
         if (samlBackend.getConfig().isAutoLoginEnabled()) {
             LoginConfiguration loginConfiguration = loginConfigurationLookup.getLoginConfiguration();
-            Cookie samlCookie = SAMLLoginTools.getSAMLCookie(httpRequest, loginConfiguration);
-            if (samlCookie == null) {
+            Cookie sessionCookie = SAMLLoginTools.getSessionCookie(httpRequest, loginConfiguration);
+            if (sessionCookie == null) {
                 return null;
             }
 
-            Session session = SAMLLoginTools.getLocalSessionForSAMLCookie(samlCookie, services.getService(SessiondService.class));
+            Session session = SAMLLoginTools.getSessionForSessionCookie(sessionCookie, services.getService(SessiondService.class));
             if (session == null) {
                 // cookie exists but no according session was found => remove it
-                Cookie toRemove = (Cookie) samlCookie.clone();
+                Cookie toRemove = (Cookie) sessionCookie.clone();
                 toRemove.setMaxAge(0);
                 httpResponse.addCookie(toRemove);
 
-                LOG.debug("Found no session for SAML auto-login cookie '{}' with value '{}'", samlCookie.getName(), samlCookie.getValue());
+                LOG.debug("Found no session for session cookie '{}' with value '{}'", sessionCookie.getName(), sessionCookie.getValue());
+                return null;
+            }
+            int expectedContextId = reservation.getContextId();
+            int expectedUserId = reservation.getUserId();
+            if (expectedContextId != session.getContextId() || expectedUserId != session.getUserId()) {
+                // wrong session
+                LOG.debug("Session {} does not match expected session for reservation {}.", session.getSessionID(), reservation.getToken());
                 return null;
             }
 
             try {
-                LOG.debug("Found session '{}' for SAML auto-login cookie '{}' with value '{}'", session.getSessionID(), samlCookie.getName(), samlCookie.getValue());
+                LOG.debug("Found session '{}' for session cookie '{}' with value '{}'", session.getSessionID(), sessionCookie.getName(), sessionCookie.getValue());
                 String hash = HashCalculator.getInstance().getHash(httpRequest, LoginTools.parseUserAgent(httpRequest), LoginTools.parseClient(httpRequest, false, loginConfiguration.getDefaultClient()));
                 SAMLLoginTools.validateSession(httpRequest, session, hash, loginConfiguration);
                 // compare against authInfo
@@ -400,7 +353,7 @@ public class SAMLLoginRequestHandler implements LoginRequestHandler {
                     }
                     return SAMLLoginTools.buildAbsoluteFrontendRedirectLocation(httpRequest, session, uiWebPath, services.getOptionalService(HostnameService.class));
                 }
-                LOG.debug("Session in SAML auto-login cookie is different to authInfo user and context");
+                LOG.debug("Session in session cookie is different to authInfo user and context");
             } catch (OXException e) {
                 LOG.debug("Ignoring SAML auto-login attempt due to failed IP or secret check", e);
             }
