@@ -1,56 +1,27 @@
 /*
+ * @copyright Copyright (c) OX Software GmbH, Germany <info@open-xchange.com>
+ * @license AGPL-3.0
  *
- *    OPEN-XCHANGE legal information
+ * This code is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU Affero General Public License as published by
+ * the Free Software Foundation, either version 3 of the License, or
+ * (at your option) any later version.
  *
- *    All intellectual property rights in the Software are protected by
- *    international copyright laws.
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU Affero General Public License for more details.
  *
+ * You should have received a copy of the GNU Affero General Public License
+ * along with OX App Suite.  If not, see <https://www.gnu.org/licenses/agpl-3.0.txt>.
  *
- *    In some countries OX, OX Open-Xchange, open xchange and OXtender
- *    as well as the corresponding Logos OX Open-Xchange and OX are registered
- *    trademarks of the OX Software GmbH group of companies.
- *    The use of the Logos is not covered by the GNU General Public License.
- *    Instead, you are allowed to use these Logos according to the terms and
- *    conditions of the Creative Commons License, Version 2.5, Attribution,
- *    Non-commercial, ShareAlike, and the interpretation of the term
- *    Non-commercial applicable to the aforementioned license is published
- *    on the web site http://www.open-xchange.com/EN/legal/index.html.
- *
- *    Please make sure that third-party modules and libraries are used
- *    according to their respective licenses.
- *
- *    Any modifications to this package must retain all copyright notices
- *    of the original copyright holder(s) for the original code used.
- *
- *    After any such modifications, the original and derivative code shall remain
- *    under the copyright of the copyright holder(s) and/or original author(s)per
- *    the Attribution and Assignment Agreement that can be located at
- *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
- *    given Attribution for the derivative code and a license granting use.
- *
- *     Copyright (C) 2016-2020 OX Software GmbH
- *     Mail: info@open-xchange.com
- *
- *
- *     This program is free software; you can redistribute it and/or modify it
- *     under the terms of the GNU General Public License, Version 2 as published
- *     by the Free Software Foundation.
- *
- *     This program is distributed in the hope that it will be useful, but
- *     WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
- *     or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
- *     for more details.
- *
- *     You should have received a copy of the GNU General Public License along
- *     with this program; if not, write to the Free Software Foundation, Inc., 59
- *     Temple Place, Suite 330, Boston, MA 02111-1307 USA
+ * Any use of the work other than as authorized under this license or copyright law is prohibited.
  *
  */
 
 package com.openexchange.saml.tools;
 
 import static com.openexchange.ajax.AJAXServlet.PARAMETER_SESSION;
-import java.util.Collection;
 import java.util.Map;
 import javax.servlet.http.Cookie;
 import javax.servlet.http.HttpServletRequest;
@@ -62,10 +33,9 @@ import com.openexchange.ajax.login.LoginConfiguration;
 import com.openexchange.ajax.login.LoginTools;
 import com.openexchange.exception.OXException;
 import com.openexchange.groupware.notify.hostname.HostnameService;
-import com.openexchange.saml.SAMLSessionParameters;
 import com.openexchange.session.Session;
+import com.openexchange.sessiond.ExpirationReason;
 import com.openexchange.sessiond.SessionExceptionCodes;
-import com.openexchange.sessiond.SessionFilter;
 import com.openexchange.sessiond.SessiondService;
 import com.openexchange.tools.servlet.http.Cookies;
 import com.openexchange.tools.servlet.http.Tools;
@@ -108,11 +78,6 @@ public class SAMLLoginTools {
      */
     public static final String ACTION_SAML_LOGOUT = "samlLogout";
 
-    /**
-     * The prefix of the auto-login cookie (<code>open-xchange-saml-</code>).
-     */
-    public static final String AUTO_LOGIN_COOKIE_PREFIX = "open-xchange-saml-";
-
     /**
      * Generates the relative redirect location to enter the web front-end directly with a session.
      *
@@ -149,39 +114,26 @@ public class SAMLLoginTools {
         return location.toString();
     }
 
-    /**
-     * Gets the {@code open-xchange-saml-<hash>} cookie from given HTTP request, if available.
-     *
-     * @param httpRequest The inbound HTTP request
-     * @param loginConfiguration The current login configuration
-     * @return The cookie or <code>null</code>
-     * @throws OXException If an unexpected error occurs
-     */
-    public static Cookie getSAMLCookie(HttpServletRequest httpRequest, LoginConfiguration loginConfiguration) throws OXException {
+    public static Cookie getSessionCookie(HttpServletRequest httpRequest, LoginConfiguration loginConfiguration) throws OXException {
         String hash = HashCalculator.getInstance().getHash(httpRequest, LoginTools.parseUserAgent(httpRequest), LoginTools.parseClient(httpRequest, false, loginConfiguration.getDefaultClient()));
         Map<String, Cookie> cookies = Cookies.cookieMapFor(httpRequest);
-        return cookies.get(SAMLLoginTools.AUTO_LOGIN_COOKIE_PREFIX + hash);
+        return cookies.get(LoginServlet.SESSION_PREFIX + hash);
     }
 
     /**
-     * Gets the node-local session object that belongs to given {@code open-xchange-saml-<hash>} cookie, if available.
+     * Gets the (possibly cluster-wide distributed) session object that belongs to given {@code open-xchange-session-<hash>} cookie, if available.
      *
-     * @param samlCookie The SAML cookie or <code>null</code>
+     * @param sessionCookie The session cookie or <code>null</code>
      * @param sessiondService The {@link SessiondService} instance to lookup the session
      * @return The session or <code>null</code> if it doesn't exist or cookie was <code>null</code>
      * @throws OXException  If an unexpected error occurs
      */
-    public static Session getLocalSessionForSAMLCookie(Cookie samlCookie, SessiondService sessiondService) throws IllegalArgumentException, OXException {
-        if (samlCookie == null) {
+    public static Session getSessionForSessionCookie(Cookie sessionCookie, SessiondService sessiondService) throws IllegalArgumentException {
+        if (sessionCookie == null) {
             return null;
         }
 
-        Collection<String> sessions = sessiondService.findSessions(SessionFilter.create("(" + SAMLSessionParameters.SESSION_COOKIE + "=" + samlCookie.getValue() + ")"));
-        if (sessions.size() > 0) {
-            return sessiondService.getSession(sessions.iterator().next());
-        }
-
-        return null;
+        return sessiondService.getSession(sessionCookie.getValue());
     }
 
     /**
@@ -194,8 +146,11 @@ public class SAMLLoginTools {
      * @throws {@link SessionExceptionCodes#SESSION_EXPIRED}
      */
     public static void validateSession(HttpServletRequest httpRequest, Session session, String cookieHash, @SuppressWarnings("unused") LoginConfiguration loginConfiguration) throws OXException {
-        if (!isValidSession(httpRequest, session, cookieHash)) {
-            throw SessionExceptionCodes.SESSION_EXPIRED.create(session.getSessionID());
+        ExpirationReason optExpirationReason = getExpirationReasonOrNull(httpRequest, session, cookieHash);
+        if (optExpirationReason != null) {
+            OXException oxe = SessionExceptionCodes.SESSION_EXPIRED.create(session.getSessionID());
+            oxe.setProperty(SessionExceptionCodes.OXEXCEPTION_PROPERTY_SESSION_EXPIRATION_REASON, optExpirationReason.getIdentifier());
+            throw oxe;
         }
     }
 
@@ -208,18 +163,28 @@ public class SAMLLoginTools {
      * @return {@code true} if the session valid, other wise {@code false}
      * @throws OXException
      */
-    public static boolean isValidSession(HttpServletRequest httpRequest, Session session, String cookieHash) throws OXException {
+    public static boolean isValidSession(HttpServletRequest httpRequest, Session session, String cookieHash) {
+        return getExpirationReasonOrNull(httpRequest, session, cookieHash) == null;
+    }
+
+    private static ExpirationReason getExpirationReasonOrNull(HttpServletRequest httpRequest, Session session, String cookieHash) {
         // IP check
         try {
             SessionUtility.checkIP(session, httpRequest.getRemoteAddr());
         } catch (OXException e) {
-            return false;
+            return ExpirationReason.IP_CHECK_FAILED;
         }
 
         // Check secret cookie
         Map<String, Cookie> cookies = Cookies.cookieMapFor(httpRequest);
         Cookie secretCookie = cookies.get(LoginServlet.SECRET_PREFIX + cookieHash);
-        return secretCookie != null && session.getSecret().equals(secretCookie.getValue());
+        if (secretCookie == null) {
+            return ExpirationReason.NO_EXPECTED_SECRET_COOKIE;
+        }
+        if (session.getSecret().equals(secretCookie.getValue()) == false) {
+            return ExpirationReason.SECRET_MISMATCH;
+        }
+        return null;
     }
 
     /**
